import UIKit
import Combine

class ViewController: UIViewController {
    var subscriptions: [AnyCancellable] = []

    override func viewDidLoad() {
        super.viewDidLoad()

        let (redView, redValue) = makeComponent(name: "Red")
        let (greenView, greenValue) = makeComponent(name: "Green")
        let (blueView, blueValue) = makeComponent(name: "Blue")

        let colorView = UIView()
        let colorSubscription = Publishers.CombineLatest3(redValue, greenValue, blueValue)
            .map { UIColor(red: $0, green: $1, blue: $2, alpha: 1.0) }
            .assign(to: \UIView.backgroundColor, on: colorView)
        subscriptions.append(colorSubscription)

        let spacerView = UIView()

        let stackView = UIStackView(arrangedSubviews: [colorView, redView, greenView, blueView, spacerView])
        stackView.axis = .vertical
        stackView.translatesAutoresizingMaskIntoConstraints = false
        view.addSubview(stackView)

        NSLayoutConstraint.activate([
            colorView.heightAnchor.constraint(equalToConstant: 200),
            view.layoutMarginsGuide.topAnchor.constraint(equalTo: stackView.topAnchor),
            view.layoutMarginsGuide.bottomAnchor.constraint(equalTo: stackView.bottomAnchor),
            view.layoutMarginsGuide.leadingAnchor.constraint(equalTo: stackView.leadingAnchor),
            view.layoutMarginsGuide.trailingAnchor.constraint(equalTo: stackView.trailingAnchor),
        ])
    }

    func makeComponent(name: String) -> (UIView, AnyPublisher<CGFloat, Never>) {
        let nameLabel = UILabel()
        nameLabel.text = name

        let textField = UITextField()
        textField.text = "0"
        textField.borderStyle = .roundedRect

        let slider = UISlider()
        slider.value = 0

        let stackView = UIStackView(arrangedSubviews: [nameLabel, textField, slider])
        stackView.axis = .horizontal
        stackView.distribution = .fillEqually

        let inputValue = NotificationCenter.default
            .publisher(for: UITextField.textDidChangeNotification, object: textField)
            .compactMap { ($0.object as? UITextField)?.text }
            .compactMap { Int($0) }
            .map { max(min($0, 255), 0) }
            .map { CGFloat(integerLiteral: $0) / 255.0 }

        let textDidEndEditingSubscription = NotificationCenter.default
            .publisher(for: UITextField.textDidEndEditingNotification, object: textField)
            .compactMap { ($0.object as? UITextField)?.text }
            .compactMap { Int($0) }
            .map { "\(max(min($0, 255), 0))" }
            .assign(to: \UITextField.text, on: textField)
        subscriptions.append(textDidEndEditingSubscription)

        let sliderValue = slider.valueChangedPublisher()
            .map { CGFloat($0) }

        let assignToInputSubscription = sliderValue.map { "\(Int($0 * 255))" }
            .assign(to: \UITextField.text, on: textField)
        subscriptions.append(assignToInputSubscription)

        let assignToSliderSubscription = inputValue.map { Float($0) }
            .assign(to: \UISlider.value, on: slider)
        subscriptions.append(assignToSliderSubscription)

        let publisher = Publishers.Merge(inputValue, sliderValue)
            .prepend(0)
            .eraseToAnyPublisher()

        return (stackView, publisher)
    }
}

extension UISlider {
    struct ValueChangedPublisher: Publisher {
        typealias Output = Float
        typealias Failure = Never

        let control: UISlider

        init(for control: UISlider) {
            self.control = control
        }

        func receive<S>(subscriber: S) where S : Subscriber, Failure == S.Failure, Output == S.Input {
            let subscription = ValueChangedSubscription(subscriber: subscriber, control: control)
            subscriber.receive(subscription: subscription)
        }
    }

    class ValueChangedSubscription<S: Subscriber>: Subscription where S.Input == Float, S.Failure == Never {
        let subscriber: S
        let control: UISlider

        init(subscriber: S, control: UISlider) {
            self.subscriber = subscriber
            self.control = control
        }

        func request(_ demand: Subscribers.Demand) {
            control.addTarget(self, action: #selector(handleAction(sender:)), for: .valueChanged)
        }

        func cancel() {
            control.removeTarget(self, action: #selector(handleAction(sender:)), for: .valueChanged)
        }

        @objc func handleAction(sender: UISlider) {
            _ = subscriber.receive(sender.value)
        }
    }

    func valueChangedPublisher() -> ValueChangedPublisher {
        ValueChangedPublisher(for: self)
    }
}
